@@ -2,14 +2,19 @@
 
 # This file is part of python-markups test suite
 # License: 3-clause BSD, see LICENSE file
-# Copyright: (C) Dmitry Shachnev, 2012-2018
+# Copyright: (C) Dmitry Shachnev, 2012-2021
 
 from markups.markdown import MarkdownMarkup, _canonicalized_ext_names
-from os import remove
-from tempfile import NamedTemporaryFile
+from os.path import join
+from tempfile import TemporaryDirectory
 import unittest
 import warnings
 
+try:
+	import pymdownx
+except ImportError:
+	pymdownx = None
+
 tables_source = \
 '''th1 | th2
 --- | ---
@@ -104,7 +109,7 @@ $$
 '''
 
 mathjax_multiline_output = \
-'''<p>
+r'''<p>
 <script type="math/tex; mode=display">
 \TeX
 \LaTeX
@@ -208,6 +213,11 @@ class MarkdownTest(unittest.TestCase):
 			tables_source).get_document_body()
 		self.assertNotIn('<table>', html)
 
+	def test_remove_extra_double(self):
+		"""Removing extra twice should not cause a crash."""
+		markup = MarkdownMarkup(extensions=['remove_extra'])
+		markup.convert('Required-Extensions: remove_extra\n')
+
 	def test_remove_extra_removes_mathjax(self):
 		markup = MarkdownMarkup(extensions=['remove_extra'])
 		html = markup.convert('$$1$$').get_document_body()
@@ -255,22 +265,92 @@ class MarkdownTest(unittest.TestCase):
 
 	def test_mathjax_asciimath(self):
 		markup = MarkdownMarkup(extensions=['mdx_math(use_asciimath=1)'])
-		converted = markup.convert('\( [[a,b],[c,d]] \)')
+		converted = markup.convert(r'\( [[a,b],[c,d]] \)')
 		body = converted.get_document_body()
 		self.assertIn('<script type="math/asciimath">', body)
 		self.assertIn('<script type="text/javascript"', converted.get_javascript())
 
-	@unittest.skipUnless(hasattr(unittest.TestCase, 'assertWarnsRegex'), 'assertWarnsRegex is not supported')
 	def test_not_loading_sys(self):
 		with self.assertWarnsRegex(ImportWarning, 'Extension "sys" does not exist.'):
 			markup = MarkdownMarkup(extensions=['sys'])
 		self.assertNotIn('sys', markup.extensions)
 
-	def test_extensions_file(self):
-		f = NamedTemporaryFile(mode="w", delete=False)
-		f.write("foo\n# bar\nbaz\n")
-		f.close()
-		markup = MarkdownMarkup()
-		extensions = markup._load_extensions_list_from_file(f.name)
-		self.assertEqual(extensions, ["foo", "baz"])
-		remove(f.name)
+	def test_extensions_txt_file(self):
+		with TemporaryDirectory() as tmpdirname:
+			txtfilename = join(tmpdirname, "markdown-extensions.txt")
+			with open(txtfilename, "w") as f:
+				f.write("foo\n# bar\nbaz(arg=value)\n")
+			markup = MarkdownMarkup(filename=join(tmpdirname, "foo.md"))
+		self.assertEqual(markup.global_extensions,
+		                 [("foo", {}), ("baz", {"arg": "value"})])
+
+	def test_extensions_yaml_file(self):
+		with TemporaryDirectory() as tmpdirname:
+			yamlfilename = join(tmpdirname, "markdown-extensions.yaml")
+			with open(yamlfilename, "w") as f:
+				f.write('- smarty:\n'
+				        '    substitutions:\n'
+				        '      left-single-quote: "&sbquo;"\n'
+				        '      right-single-quote: "&lsquo;"\n'
+				        '    smart_dashes: False\n'
+				        '- toc:\n'
+				        '    permalink: True\n'
+				        '    separator: "_"\n'
+				        '    toc_depth: 3\n'
+				        '- sane_lists\n')
+			markup = MarkdownMarkup(filename=join(tmpdirname, "foo.md"))
+		self.assertEqual(
+			markup.global_extensions,
+			[("smarty", {"substitutions": {"left-single-quote": "&sbquo;",
+			                               "right-single-quote": "&lsquo;"},
+			  "smart_dashes": False}),
+			 ("toc", {"permalink": True, "separator": "_", "toc_depth": 3}),
+			 ("sane_lists", {}),
+			])
+		converted = markup.convert("'foo' -- bar")
+		body = converted.get_document_body()
+		self.assertEqual(body, '<p>&sbquo;foo&lsquo; -- bar</p>\n')
+
+	def test_extensions_yaml_file_invalid(self):
+		with TemporaryDirectory() as tmpdirname:
+			yamlfilename = join(tmpdirname, "markdown-extensions.yaml")
+			with open(yamlfilename, "w") as f:
+				f.write('[this is an invalid YAML file')
+			with self.assertWarns(SyntaxWarning) as cm:
+				MarkdownMarkup(filename=join(tmpdirname, "foo.md"))
+			self.assertIn("Failed parsing", str(cm.warning))
+			self.assertIn("expected ',' or ']'", str(cm.warning))
+
+	def test_codehilite(self):
+		markup = MarkdownMarkup(extensions=["codehilite"])
+		converted = markup.convert('    :::python\n    import foo')
+		stylesheet = converted.get_stylesheet()
+		self.assertIn(".codehilite .k {", stylesheet)
+		body = converted.get_document_body()
+		self.assertIn('<div class="codehilite">', body)
+
+	def test_codehilite_custom_class(self):
+		markup = MarkdownMarkup(extensions=["codehilite(css_class=myclass)"])
+		converted = markup.convert('    :::python\n    import foo')
+		stylesheet = converted.get_stylesheet()
+		self.assertIn(".myclass .k {", stylesheet)
+		body = converted.get_document_body()
+		self.assertIn('<div class="myclass">', body)
+
+	@unittest.skipIf(pymdownx is None, "pymdownx module is not available")
+	def test_pymdownx_highlight(self):
+		markup = MarkdownMarkup(extensions=["pymdownx.highlight"])
+		converted = markup.convert('    import foo')
+		stylesheet = converted.get_stylesheet()
+		self.assertIn(".highlight .k {", stylesheet)
+		body = converted.get_document_body()
+		self.assertIn('<div class="highlight">', body)
+
+	@unittest.skipIf(pymdownx is None, "pymdownx module is not available")
+	def test_pymdownx_highlight_custom_class(self):
+		markup = MarkdownMarkup(extensions=["pymdownx.highlight(css_class=myclass)"])
+		converted = markup.convert('    import foo')
+		stylesheet = converted.get_stylesheet()
+		self.assertIn(".myclass .k {", stylesheet)
+		body = converted.get_document_body()
+		self.assertIn('<div class="myclass">', body)
