@@ -2,33 +2,43 @@
 
 # This file is part of python-markups test suite
 # License: 3-clause BSD, see LICENSE file
-# Copyright: (C) Dmitry Shachnev, 2012-2018
+# Copyright: (C) Dmitry Shachnev, 2012-2021
 
 import unittest
 from markups import ReStructuredTextMarkup
 
-basic_text = \
-'''Hello, world!
+basic_text = '''\
+Hello, world!
 =============
 
 Some subtitle
 ~~~~~~~~~~~~~
 
-This is an example **reStructuredText** document.'''
+This is an example **reStructuredText** document.
+'''
+
+basic_html = '''\
+<main id="hello-world">
+<h1 class="title" data-posmap="2">Hello, world!</h1>
+<p class="subtitle" id="some-subtitle">Some subtitle</p>
+<p data-posmap="7">This is an example <strong>reStructuredText</strong> document.</p>
+</main>
+'''
 
 @unittest.skipUnless(ReStructuredTextMarkup.available(), 'Docutils not available')
 class ReStructuredTextTest(unittest.TestCase):
 	def test_basic(self):
+		import docutils
 		markup = ReStructuredTextMarkup()
 		converted = markup.convert(basic_text)
 		text = converted.get_document_body()
 		title = converted.get_document_title()
 		stylesheet = converted.get_stylesheet()
-		text_expected = ('<div class="document" id="hello-world">\n'
-			'<h1 class="title">Hello, world!</h1>\n'
-			'<p class="subtitle" id="some-subtitle">Some subtitle</p>\n'
-			'<p>This is an example <strong>reStructuredText</strong> document.</p>\n'
-			'</div>\n')
+		text_expected = basic_html
+		if docutils.__version_info__[:2] < (0, 17):
+			# docutils uses <main> tag since revision 8474
+			text_expected = text_expected.replace('<main', '<div class="document"')
+			text_expected = text_expected.replace('</main>', '</div>')
 		title_expected = 'Hello, world!'
 		self.assertEqual(text_expected, text)
 		self.assertEqual(title_expected, title)
@@ -56,9 +66,17 @@ class ReStructuredTextTest(unittest.TestCase):
 		body = markup.convert('`').get_document_body() # unclosed role
 		self.assertNotIn('system-message', body)
 
+	def test_errors_severe(self):
+		markup = ReStructuredTextMarkup(settings_overrides={'warning_stream': False})
+		text = "***************\nfaulty headline"
+		# The following line should not raise SystemMessage exception
+		body = markup.convert(text).get_document_body()
+		self.assertIn("system-message", body)
+		self.assertIn("Incomplete section title.", body)
+
 	def test_whole_html(self):
 		markup = ReStructuredTextMarkup()
-		text = basic_text + "\n\n.. math::\n   \sin \varphi"
+		text = basic_text + "\n\n.. math::\n   \\sin \\varphi"
 		html = markup.convert(text).get_whole_html()
 		self.assertIn("<title>Hello, world!</title>", html)
 		self.assertIn('<style type="text/css">', html)
